package model;

import java.math.*;
import java.net.*;
import java.util.*;

public class BambooParser {
	// private static final String URL =
	// "http://smile.poosan.net/zuozuomu/bamboo.html";

	private static final int RADIX = 32;

	private final BigInteger _7bit = new BigInteger("127");

	private final BigInteger _6bit = new BigInteger("63");

	public Cell[][] parseMap(URL url) {
		String query = url.getQuery();
		if (query != null) {
			return parseMap(query);
		} else {
			Cell[][] map = new Cell[20][20];
			for (int i = 0; i < 20; i++) {
				for (int j = 0; j < 20; j++) {
					map[i][j] = new Cell(Cell.KARA);
				}
			}
			return map;
		}
	}

	protected Cell[][] parseMap(String query) {
		/* size */
		StringTokenizer parser = new StringTokenizer(query, "c=x&r(,) ");
		String kind = parser.nextToken(); // CR[̑O̕
		if (kind.equals("size")) {
			return parse1(parser);
		} else if (kind.equals("b")) {
			return parse3(parser);
		} else {
			assert (false);
			return null;
		}
	}

	private Cell[][] parse1(StringTokenizer parser) {
		int width = Integer.parseInt(parser.nextToken());
		int height = Integer.parseInt(parser.nextToken());

		Cell[][] map = new Cell[height][width];

		int i = (map[0].length - width) / 2;
		int j = (map.length - height) / 2;

		/* }X */
		for (int y = 0; y < height; y++) {
			for (int x = 0; x < width; x++) {
				// map[j + y][i + x].setNumber(LayerList.MASU);
				map[j + y][i + x] = new Cell(Cell.MASU);
			}
		}

		/* V} */
		while (parser.hasMoreTokens()) {
			int y = Integer.parseInt(parser.nextToken());
			int x = Integer.parseInt(parser.nextToken());

			// map[y + j - 1][x + i - 1] =
			// -Integer.parseInt(parser.nextToken());
			map[y + j - 1][x + i - 1].setNumber(-Integer.parseInt(parser
					.nextToken()));
		}

		return map;
	}

	protected Cell[][] parse3(StringTokenizer parser) {
		String all = parser.nextToken("="); // CR[̕
		int width = parseInt(all.charAt(0));
		System.out.println("width=" + width);
		int height = parseInt(all.charAt(1));
		System.out.println("height=" + height);
		String problem = all.substring(2);
		System.out.println("problem=" + problem);
		int[][] p = toProblem64(problem, toBit2(width), toBit2(height));
		return toMap(p, width, height);
	}

	protected Cell[][] toMap(final int[][] p, final int width, final int height) {
		Cell[][] map = new Cell[height][width];
		for(int i = 0; i < height; i++){
			for(int j = 0; j < width; j++){
				map[i][j] = new Cell(Cell.MASU);
			}
		}
		// p[][] is problem length * 3
		for (int i = 0; i < p.length; i++) {
			int x = p[i][0];
			int y = p[i][1];
			int shima = p[i][2];
			map[y - 1][x - 1].setNumber(-shima);
		}
		return map;
	}

	public String parse1(Cell[][] cells, int x, int y, int saveWidth,
			int saveHeight) {
		StringBuffer url = new StringBuffer();
		for (int j = 0; j < saveHeight; j++) {
			for (int i = 0; i < saveWidth; i++) {
				// if (cells[j + y][i + x] < 0) {
				if (cells[j + y][i + x].compareTo(0) < 0) {
					url.append("&r" + (j + 1) + "c" + (i + 1) + "="
					// + (-cells[j + y][i + x]));
							+ (-cells[j + y][i + x].getNumber()));
				}
			}
		}
		return url.toString();
	}

	public int toBit(int number) {
		int bit = 0;
		for (int i = 1; i <= number; i *= 2) {
			bit++;
		}
		return bit;
	}

	public int toBit2(int number) {
		int bit = 0;
		for (int i = 1; i < number; i *= 2) {
			bit++;
		}
		return bit;
	}

	public int toInteger(String string) {
		return Integer.parseInt(string, RADIX);
	}

	public BigInteger toInteger64(String string) {
		BigInteger num = BigInteger.ZERO;
		for (int i = 0; i < string.length(); i++) {
			if (isWhite(string.charAt(i))) {
				continue;
			}
			num = num.shiftLeft(6).add(parseBigInt(string.charAt(i)));
		}
		return num;
	}

	private String toStringFromBigInteger(BigInteger big) {
		StringBuffer buf = new StringBuffer();
		while (!big.equals(BigInteger.ZERO)) {
			buf.insert(0, toCharFromBigInteger(big.and(_6bit)));
			big = big.shiftRight(6);
		}
		return buf.toString();
	}

	private char toCharFromBigInteger(final BigInteger num) {
		return toCharFromInteger(num.intValue());
	}

	private char toCharFromInteger(final int number) {
		if (0 <= number && number <= 9) {
			return (char) ('0' + number);
		} else if (10 <= number && number <= 35) {
			return (char) (number + 'a' - 10);
		} else if (36 <= number && number <= 61) {
			return (char) (number + 'A' - 36);
		} else if (number == 62) {
			return '-';
		} else if (number == 63) {
			return '_';
		} else {
			assert (false);
			return 0;
		}
	}

	public int[] toArray(String string, final int bit1, final int bit2) {
		int number = Integer.parseInt(string, RADIX);
		String bit = Integer.toBinaryString(number);
		int lastIndex = bit.length() - 7;
		int middleIndex = lastIndex - bit2;
		String last = bit.substring(lastIndex, bit.length());
		int shima = Integer.parseInt(last, 2);
		String middle = bit.substring(middleIndex, lastIndex);
		int y = Integer.parseInt(middle, 2);
		String first = bit.substring(0, middleIndex);
		int x = Integer.parseInt(first, 2);
		return new int[] { x, y, shima };
	}

	public int[][] toProblem(final String string, final int bit1, final int bit2) {
		int number = bit1 + bit2 + 7;
		int sub = number / 5;
		int split = string.length() / sub;
		int[][] p = new int[split][3];
		for (int i = 0; i < split; i++)
			p[i] = toArray(string.substring(sub * i, sub * (i + 1)), bit1, bit2);
		return p;
	}

	public int[][] toProblem64(final String string, final int bit1,
			final int bit2) {
		List<BigInteger> list = new ArrayList<BigInteger>();
		final BigInteger _bit2 = new BigInteger("" + (pow(2, bit2) - 1));
		final BigInteger _bit1 = new BigInteger("" + (pow(2, bit1) - 1));
		int count = 0;
		BigInteger num = toInteger64(string);
		System.out.println("num=" + num);
		while (true) {
			BigInteger problem = num.and(_7bit);
			if (problem.equals(BigInteger.ZERO)) {
				break;
			}
			// System.out.println("problem=" + problem);
			list.add(problem);
			num = num.shiftRight(7);
			BigInteger column = num.and(_bit2);
			// System.out.println("column=" + column);
			list.add(column);
			num = num.shiftRight(bit2);
			BigInteger row = num.and(_bit1);
			// System.out.println("row=" + row);
			list.add(row);
			num = num.shiftRight(bit1);
			count++;
		}
		int[][] p = new int[count][3];
		for (int i = 0; i < count; i++) {
			p[i][2] = list.get(3 * i).intValue();
			p[i][1] = list.get(3 * i + 1).intValue() + 1;
			p[i][0] = list.get(3 * i + 2).intValue() + 1;
		}
		return p;
	}

	public Cell[][] toProblemFromQuery(String string) {
		return parseMap(string);
	}

	public String toStringFromArray(final int[] p, final int bit1,
			final int bit2) {
		int total = p[0] * pow(2, bit2 + 7) + p[1] * pow(2, 7) + p[2];
		return Integer.toString(total, RADIX);
	}

	private int pow(final int e, final int r) {
		int x = 1;
		for (int i = 0; i < r; i++) {
			x *= e;
		}
		return x;
	}

	public String toStringFromProbrem(final int[][] p, final int bit1,
			final int bit2) {
		StringBuffer buf = new StringBuffer();
		for (int i = 0; i < p.length; i++)
			buf.append(toStringFromArray(p[i], bit1, bit2));
		return buf.toString();
	}

	public String toStringFromProbremB(final int[][] p, final int bit1,
			final int bit2) {
		BigInteger num = BigInteger.ZERO;
		for (int i = 0; i < p.length; i++) {
			num = num.shiftLeft(bit1).add(new BigInteger("" + (p[i][0] - 1)));
			num = num.shiftLeft(bit2).add(new BigInteger("" + (p[i][1] - 1)));
			num = num.shiftLeft(7).add(new BigInteger("" + p[i][2]));
		}
		return toStringFromBigInteger(num);
	}

	protected String toStringFromList(final List<int[]> p, final int bit1,
			final int bit2) {
		BigInteger num = BigInteger.ZERO;
		for (int[] i : p) {
			num = num.shiftLeft(bit1).add(new BigInteger("" + (i[0] - 1)));
			num = num.shiftLeft(bit2).add(new BigInteger("" + (i[1] - 1)));
			num = num.shiftLeft(7).add(new BigInteger("" + i[2]));
		}
		return toStringFromBigInteger(num);
	}

	public String toQueryFromProbrem(final int[][] p, final int i, final int j) {
		int bit1 = toBit(i);
		int bit2 = toBit(j);
		StringBuffer buf = new StringBuffer("a=");
		buf.append(i);
		buf.append("x");
		buf.append(j);
		buf.append("_");
		buf.append(toStringFromProbrem(p, bit1, bit2));
		return buf.toString();
	}

	public String toQueryFromProbremB(final int[][] p, final int i, final int j) {
		int bit1 = toBit2(i);
		int bit2 = toBit2(j);
		StringBuffer buf = new StringBuffer("b=");
		buf.append(toCharFromInteger(i));
		buf.append(toCharFromInteger(j));
		buf.append(toStringFromProbremB(p, bit1, bit2));
		return buf.toString();
	}

	protected String toQueryFromList(final List<int[]> p, final int i,
			final int j) {
		final int bit1 = toBit2(i);
		final int bit2 = toBit2(j);
		StringBuffer buf = new StringBuffer("b=");
		buf.append(toCharFromInteger(i));
		buf.append(toCharFromInteger(j));
		buf.append(toStringFromList(p, bit1, bit2));
		return buf.toString();
	}

	public String toUrlFromProbrem(final String url, final int[][] p,
			final int width, final int height) {
		return url + "?" + toQueryFromProbrem(p, width, height);
	}

	public String toUrlFromProbremB(final String url, final int[][] p,
			final int width, final int height) {
		return url + "?" + toQueryFromProbremB(p, width, height);
	}

	public String toUrlFromList(final String url, final List<int[]> p,
			final int width, final int height) {
		return url + "?" + toQueryFromList(p, width, height);
	}

	public String toUrlFromModel(BambooModel model) {
		LayerList list = model.getLayerList();
		SaveController saveController = model.getSaveController();
		saveController.setSquare();
		int x = saveController.getSaveX();
		int y = saveController.getSaveY();
		int width = saveController.getSaveWidth();
		int height = saveController.getSaveHeight();
		Cell[][] map = list.getMap();
		String url = saveController.getUrlFromFile();
		return toUrlFromMap(url, map, x, y, width, height);
	}

	public String toUrlFromModelB(final BambooModel model) {
		LayerList list = model.getLayerList();
		SaveController saveController = model.getSaveController();
		saveController.setSquare();
		int x = saveController.getSaveX();
		int y = saveController.getSaveY();
		int width = saveController.getSaveWidth();
		int height = saveController.getSaveHeight();
		Cell[][] map = list.getMap();
		String url = saveController.getUrlFromFile();
		return toUrlFromMapB(url, map, x, y, width, height);
	}

	private String toUrlFromMap(final String url, Cell[][] map, int x, int y,
			int width, int height) {
		StringBuffer buf = new StringBuffer(url);
		buf.append("?a=");
		buf.append(width);
		buf.append("x");
		buf.append(height);
		buf.append("_");
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				// if (map[i + y][j + x] < 0) {
				if (map[i + y][j + x].compareTo(0) < 0) {
					int[] p = new int[3];
					p[0] = j + 1;
					p[1] = i + 1;
					// p[2] = -map[i + y][j + x];
					p[2] = -map[i + y][j + x].getNumber();
					buf
							.append(toStringFromArray(p, toBit(width),
									toBit(height)));
				}
			}
		}
		return buf.toString();
	}

	private String toUrlFromMapB(final String url, Cell[][] map, int x, int y,
			int width, int height) {
		List<int[]> list = new ArrayList<int[]>();
		for (int i = 0; i < height; i++) {
			for (int j = 0; j < width; j++) {
				if (map[i + y][j + x].compareTo(0) < 0) {
					int[] p = new int[3];
					p[0] = j + 1;
					p[1] = i + 1;
					p[2] = -map[i + y][j + x].getNumber();
					list.add(p);
				}
			}
		}
		return toUrlFromList(url, list, width, height);
	}

	public int parseInt(final char c) {
		if ('0' <= c && c <= '9') {
			return c - '0';
		} else if ('a' <= c && c <= 'z') {
			return c - 'a' + 10;
		} else if ('A' <= c && c <= 'Z') {
			return c - 'A' + 36;
		} else if (c == '-') {
			return 62;
		} else if (c == '_') {
			return 63;
		} else {
			// assert (false);
			return -1;
		}
	}

	private boolean isWhite(final char c) {
		return parseInt(c) == -1;
	}

	public BigInteger parseBigInt(final char c) {
		return new BigInteger("" + parseInt(c));
	}
}
