package model;

import java.util.*;

import junit.framework.*;

public class BambooParserTest extends TestCase {
	private BambooParser parser = new BambooParser();

	public void testToBit() {
		assertEquals(3, parser.toBit(7));
		assertEquals(4, parser.toBit(8));
		assertEquals(4, parser.toBit(9));
	}

	public void testToIntegerFromString() {
		assertEquals(320, parser.toInteger("a0"));
		int[] probrem = parser.toArray("4cf", 4, 4);
		int[] p = { 2, 3, 15 };
		for (int i = 0; i < 3; i++)
			assertEquals(p[i], probrem[i]);
		int[][] p2 = { { 2, 3, 15 }, { 4, 2, 6 } };
		for (int i = 0; i < p2.length; i++)
			for (int j = 0; j < p2[i].length; j++) {
				assertEquals(p2[i][j], parser.toProblem("4cf886", 4, 4)[i][j]);
			}
		Cell[][] map = parser.toProblemFromQuery("a=9x9_4cf886");
		assertEquals(9, map.length);
		assertEquals(9, map[0].length);
		assertEquals(-15, map[2][1]);
		assertEquals(-6, map[1][3]);
	}

	public void testToIntegerFromStringB() {
		assertEquals(36, parser.parseInt('A'));
		assertEquals(2312, parser.toInteger64("A8").intValue());
		int[][] p2 = { { 1, 1, 1 } };
		for (int i = 0; i < p2.length; i++)
			for (int j = 0; j < p2[i].length; j++) {
				assertEquals(p2[i][j], parser.toProblem64("1", 2, 2)[i][j]);
			}
		int[][] p3 = { { 1, 3, 2 }, { 1, 1, 1 } };
		for (int i = 0; i < p3.length; i++)
			for (int j = 0; j < p3[i].length; j++) {
				assertEquals(p3[i][j], parser.toProblem64("A2", 2, 2)[i][j]);
			}

		Cell[][] map = parser.toProblemFromQuery("b=44A2");
		assertEquals(4, map.length);
		assertEquals(4, map[0].length);
		assertEquals(-1, map[0][0]);
		assertEquals(-2, map[2][0]);
	}

	public void testToStringFromArray() {
		int[][] p = { { 2, 3, 15 }, { 4, 2, 6 } };
		assertEquals("4cf886", parser.toStringFromProbrem(p, 4, 4));
		assertEquals("a=9x9_4cf886", parser.toQueryFromProbrem(p, 9, 9));
	}

	public void testToStringFromArrayB() {
		int[][] p = { { 1, 1, 1 }, { 1, 3, 2 } };
		List<int[]> list = new ArrayList<int[]>();
		list.add(p[0]);
		list.add(p[1]);
		assertEquals("A2", parser.toStringFromProbremB(p, 2, 2));
		assertEquals("b=44A2", parser.toQueryFromProbremB(p, 4, 4));
	}

}
