package zuozuomu.img;

import java.awt.*;
import java.awt.image.*;
import java.io.*;

import javax.imageio.*;

import controller.BambooController;

import model.*;

import ui.*;

public class ImageMaker {
	private BufferedImage[] shima = new BufferedImage[9];
	private BufferedImage leftBamboo, rightBamboo, topBamboo, bottomBamboo,
			horizontal, vertical;
	private BufferedImage[] half = new BufferedImage[10];
	private BufferedImage[] shimaOriginal = new BufferedImage[9];
	private BufferedImage leftBambooOriginal, rightBambooOriginal,
			topBambooOriginal, bottomBambooOriginal, horizontalOriginal,
			verticalOriginal;
	private BufferedImage[] halfOriginal = new BufferedImage[10];
	private BambooController controller;

	public ImageMaker(BambooController controller) {
		this.controller = controller;
		int size = controller.getSizeOfCell() + 1;
		try {
			Image image;
			for (int i = 0; i < shima.length; i++) {
				shima[i] = new BufferedImage(size, size,
						BufferedImage.TYPE_INT_ARGB);
				shimaOriginal[i] = ImageIO.read(getClass().getResource(
						"shima" + (i + 1) + ".png"));
				image = shimaOriginal[i].getScaledInstance(
						controller.getSizeOfCell() + 1, controller.getSizeOfCell() + 1,
						Image.SCALE_SMOOTH);
				shima[i].getGraphics().drawImage(image, 0, 0, null);
			}
			leftBamboo = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			leftBambooOriginal = ImageIO.read(getClass().getResource(
					"leftBamboo.png"));
			image = leftBambooOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			leftBamboo.getGraphics().drawImage(image, 0, 0, null);

			rightBamboo = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			rightBambooOriginal = ImageIO.read(getClass().getResource(
					"rightBamboo.png"));
			image = rightBambooOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			rightBamboo.getGraphics().drawImage(image, 0, 0, null);

			topBamboo = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			topBambooOriginal = ImageIO.read(getClass().getResource(
					"topBamboo.png"));
			image = topBambooOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			topBamboo.getGraphics().drawImage(image, 0, 0, null);

			bottomBamboo = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			bottomBambooOriginal = ImageIO.read(getClass().getResource(
					"bottomBamboo.png"));
			image = bottomBambooOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			bottomBamboo.getGraphics().drawImage(image, 0, 0, null);

			vertical = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			verticalOriginal = ImageIO.read(getClass().getResource(
					"verticalBamboo.png"));
			image = verticalOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			vertical.getGraphics().drawImage(image, 0, 0, null);

			horizontal = new BufferedImage(size, size,
					BufferedImage.TYPE_INT_ARGB);
			horizontalOriginal = ImageIO.read(getClass().getResource(
					"horizontalBamboo.png"));
			image = horizontalOriginal.getScaledInstance(size, size,
					Image.SCALE_SMOOTH);
			horizontal.getGraphics().drawImage(image, 0, 0, null);

			// 2P^p̉摜
			for (int i = 0; i < half.length; i++) {
				half[i] = new BufferedImage(size, size,
						BufferedImage.TYPE_INT_ARGB);
				halfOriginal[i] = ImageIO.read(getClass().getResource(
						"half" + i + ".png"));
				image = halfOriginal[i].getScaledInstance((size + 1) / 2, size,
						Image.SCALE_SMOOTH);
				half[i].getGraphics().drawImage(image, 0, 0, null);
			}

		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	private void drawCursor(Graphics g) {
		g.setColor(Color.RED);
		if (!controller.isRegion()) {
			g.drawRect(controller.getSizeOfCell() * (controller.getX() - 1),
					controller.getSizeOfCell() * (controller.getY() - 1),
					controller.getSizeOfCell(), controller.getSizeOfCell());
		} else if (controller.isRegion()) {
			int minX = Math.min(controller.getX(), controller.getX0());
			int maxX = Math.max(controller.getX(), controller.getX0());
			int minY = Math.min(controller.getY(), controller.getY0());
			int maxY = Math.max(controller.getY(), controller.getY0());
			for (int i = minX - 1; i <= maxX; i++) {
				g.drawLine(controller.getSizeOfCell() * i, controller.getSizeOfCell()
						* (minY - 1), controller.getSizeOfCell() * i,
						controller.getSizeOfCell() * maxY);
			}
			for (int j = minY - 1; j <= maxY; j++) {
				g
						.drawLine(controller.getSizeOfCell() * (minX - 1),
								controller.getSizeOfCell() * j,
								controller.getSizeOfCell() * maxX,
								controller.getSizeOfCell() * j);
			}
		}
	}

	private void makeMasu(Graphics g, int i, int j, int size) {
		g.drawRect(size * i, size * j, size, size);
	}

	public void makeMap(Graphics g) {
		g.setColor(Color.WHITE);
		g.fillRect(0, 0, controller.getPanelWidth(), controller.getPanelHeight());
		g.setColor(Color.BLACK);
		for (int layer = 0; layer < controller.getLayer() + 1; layer++)
			for (int j = 1; j < controller.getCellsHeight() - 1; j++) {
				for (int i = 1; i < controller.getCellsWidth() - 1; i++) {
					drawShima(g, layer, i, j, half, shima,
							controller.getSizeOfCell());
					switch (controller.getMap(layer, i, j)) {
					case Cell.HIDARI:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						g.drawImage(leftBamboo,
								controller.getSizeOfCell() * (i - 1),
								controller.getSizeOfCell() * (j - 1), null);
						break;
					case Cell.YOKO:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						g.drawImage(horizontal,
								controller.getSizeOfCell() * (i - 1),
								controller.getSizeOfCell() * (j - 1), null);
						break;
					case Cell.MIGI:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						g.drawImage(rightBamboo, controller.getSizeOfCell()
								* (i - 1), controller.getSizeOfCell() * (j - 1),
								null);
						break;
					case Cell.UE:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						g.drawImage(topBamboo, controller.getSizeOfCell() * (i - 1),
								controller.getSizeOfCell() * (j - 1), null);
						break;
					case Cell.TATE:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						g.drawImage(vertical, controller.getSizeOfCell() * (i - 1),
								controller.getSizeOfCell() * (j - 1), null);
						break;
					case Cell.SHITA:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						g.drawImage(bottomBamboo, controller.getSizeOfCell()
								* (i - 1), controller.getSizeOfCell() * (j - 1),
								null);
						break;
					case Cell.SHIMA:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						int center = (controller.getSizeOfCell() + 1 - controller.getSizeOfCell() / 8) / 2;
						g.fillRect(controller.getSizeOfCell() * (i - 1) + center,
								controller.getSizeOfCell() * (j - 1) + center,
								controller.getSizeOfCell() / 8,
								controller.getSizeOfCell() / 8);
						break;
					case Cell.MASU:
						makeMasu(g, i - 1, j - 1, controller.getSizeOfCell());
						break;
					}
				}
			}

		drawCursor(g);
	}

	private void drawShima(Graphics g, int layer, int i, int j,
			BufferedImage[] image1, BufferedImage[] image2, int size) {
		drawShima(g, layer, i - 1, j - 1, 1, 1, image1, image2, size);
	}

	private void drawShima(Graphics g, int layer, int i, int j, int left,
			int top, BufferedImage[] image1, BufferedImage[] image2, int size) {
		if (controller.getMap(layer, i + left, j + top) == Cell.ZERO) {
			// 0V}̂Ƃ
			makeMasu(g, i, j, size);
			g.drawOval(i * size + size / 8, j * size + size / 8, size - size
					/ 4, size - size / 4);
		} else if (controller.getMap(layer, i + left, j + top) == Cell.BATSU) {
			// oĉƂ
			makeMasu(g, i, j, size);
			g.drawLine(i * size + size / 8, j * size + size / 8, (i + 1) * size
					- size / 8, (j + 1) * size - size / 8);
			g.drawLine(i * size + size / 8, (j + 1) * size - size / 8, (i + 1)
					* size - size / 8, j * size + size / 8);
		} else if (controller.getMap(layer, i + left, j + top) <= -10) {
			// 2P^V}̂Ƃ
			makeMasu(g, i, j, size);
			g.drawImage(
					image1[-controller.getMap(layer, i + left, j + top) / 10],
					size * i, size * j, null);
			g.drawImage(
					image1[-controller.getMap(layer, i + left, j + top) % 10],
					size * i + size / 2, size * j, null);
		} else if (-10 < controller.getMap(layer, i + left, j + top)
				&& controller.getMap(layer, i + left, j + top) < 0) {
			// 1P^V}̂Ƃ
			g.drawImage(
					image2[-controller.getMap(layer, i + left, j + top) - 1],
					size * i, size * j, null);
		}
	}

	public void write(Graphics g, int left, int right, int top, int bottom) {
		g.setColor(Color.BLACK);
		for (int layer = 0; layer < controller.getLayerSize(); layer++)
			for (int j = 0; j <= bottom - top; j++) {
				for (int i = 0; i <= right - left; i++) {
					drawShima(g, layer, i, j, left, top, halfOriginal,
							shimaOriginal, BambooView.sizeOfOriginal);
					switch (controller.getMap(layer, i + left, j + top)) {
					case 2:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.drawImage(leftBambooOriginal,
								BambooView.sizeOfOriginal * i,
								BambooView.sizeOfOriginal * j, null);
						break;
					case 3:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.drawImage(horizontalOriginal,
								BambooView.sizeOfOriginal * i,
								BambooView.sizeOfOriginal * j, null);
						break;
					case 4:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.drawImage(rightBambooOriginal,
								BambooView.sizeOfOriginal * i,
								BambooView.sizeOfOriginal * j, null);
						break;
					case 5:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.drawImage(topBambooOriginal,
								BambooView.sizeOfOriginal * i,
								BambooView.sizeOfOriginal * j, null);
						break;
					case 6:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.drawImage(verticalOriginal, BambooView.sizeOfOriginal
								* i, BambooView.sizeOfOriginal * j, null);
						break;
					case 7:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.drawImage(bottomBambooOriginal,
								BambooView.sizeOfOriginal * i,
								BambooView.sizeOfOriginal * j, null);
						break;
					case 8:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						g.fillRect(BambooView.sizeOfOriginal * i + 11,
								BambooView.sizeOfOriginal * j + 11, 3, 3);
						break;
					case 0:
						makeMasu(g, i, j, BambooView.sizeOfOriginal);
						break;

					}
				}
			}
	}
}
