package zuozuomu.watermark;

import java.awt.image.*;
import java.io.*;
import java.util.*;

import javax.imageio.*;
import javax.imageio.stream.*;

import model.Cell;

public class JikkenImage {
	private BufferedImage bufImage;

	public JikkenImage(int width, int height) {
		bufImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
	}

	public JikkenImage(String fileName) {
		try {
			File inputFile = new File(fileName);
			bufImage = ImageIO.read(inputFile);
			if (bufImage == null)
				throw new IOException("Ԃ摜t@CႠ܂");
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public JikkenImage(File file) {
		this(file.getPath());
	}

	public JikkenImage(BufferedImage image) {
		this.bufImage = image;
	}

	public int getWidth() {
		return bufImage.getWidth();
	}

	public int getHeight() {
		return bufImage.getHeight();
	}

	public int getPixelValue(int i, int j, int c) {
		assert (c >= 0);
		assert (c < 3);
		if (i < 0 || i >= getWidth() || j < 0 || j >= getHeight()) {
			return 0;
		}
		int rgb = bufImage.getRGB(i, j);
		switch (c) {
		case 0:
			int r = (rgb >> 16) & 0x000000ff;
			return r;
		case 1:
			int g = (rgb >> 8) & 0x000000ff;
			return g;
		case 2:
			int b = (rgb >> 0) & 0x000000ff;
			return b;
		default:
			assert (false);
			return 0;
		}
	}

	public void setPixelValue(int i, int j, int c, int newPixelValue) {
		assert (c >= 0);
		assert (c < 3);
		int rgb = bufImage.getRGB(i, j);
		int r = (rgb >> 16) & 0x000000ff;
		int g = (rgb >> 8) & 0x000000ff;
		int b = (rgb >> 0) & 0x000000ff;
		switch (c) {
		case 0: // r
			r = newPixelValue;
			break;
		case 1: // g
			g = newPixelValue;
			break;
		case 2: // b
			b = newPixelValue;
			break;
		default:
			// System.exit(1);
			assert (false);
		}

		rgb = (r << 16) | (g << 8) | b;
		bufImage.setRGB(i, j, rgb);
	}

	public void writeJpegFile(String fileName, String formatName) {

		try {
			// Get Writer and set compression
			Iterator<ImageWriter> iter = ImageIO
					.getImageWritersByFormatName(formatName);
			if (iter.hasNext()) {
				ImageWriter writer = iter.next();
				ImageWriteParam iwp = writer.getDefaultWriteParam();
				if (iwp.canWriteCompressed()) {
					iwp.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
					iwp.setCompressionQuality(1.0f);
				}
				String newName = fileName;
				if (!newName.endsWith("." + formatName)) {
					newName += "." + formatName;
				}
				File outFile = new File(newName);
				FileImageOutputStream output = new FileImageOutputStream(
						outFile);
				writer.setOutput(output);
				IIOImage image =
				// new IIOImage(input, null, null);
				new IIOImage(bufImage, null, null);
				// System.out.println("Writing " + 1.0 + "%");
				writer.write(null, image, iwp);
			}
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	public void setSegmentation() {
		for (int i = 0; i < getWidth(); i++) {
			for (int j = 0; j < getHeight(); j++) {
				int color = getPixelGrayValue(i, j);
				setPixelSegmentationValue(i, j, color);
			}
		}
	}

	public void writeJpegFile(File file, String format) {
		writeJpegFile(file.getPath(), format);
	}

	public void setPixelGrayValue(int i, int j, int newPixelValue) {
		int rgb = (newPixelValue << 16) | (newPixelValue << 8) | newPixelValue;
		bufImage.setRGB(i, j, rgb);
	}

	public void setPixelSegmentationValue(int i, int j, int newPixelValue) {
		int color = newPixelValue < 128 ? 0 : 255;
		setPixelGrayValue(i, j, color);
	}

	public int getPixelSegmentationValue(int i, int j) {
		int gray = getPixelGrayValue(i, j);
		return gray < 128 ? 0 : 255;
	}

	public int getPixelGrayValue(int i, int j) {
		int rgb = bufImage.getRGB(i, j);
		int r = (rgb >> 16) & 0x000000ff;
		int g = (rgb >> 8) & 0x000000ff;
		int b = (rgb >> 0) & 0x000000ff;
		return (int) Math.round((r + g + b) / 3.0);
	}

	public void embedData(int i, int j, int data) {
		assert (data == 0 || data == 1);
		if (data == 0)
			return;
		int color = getPixelGrayValue(i, j);
		if (Math.abs(color - 255) <= 1)
			setPixelGrayValue(i, j, 250);
		else
			setPixelGrayValue(i, j, 5);
	}

	public void embedIntData(int i, int j, byte data) {
		int[] dataArray = ConvertUtil.convertToBitArrayFromByte(data);
		for (int s = 0; s < 8; s++) {
			embedData(i + s, j, dataArray[s]);
		}
	}

	public void embedIntArray(int i, int j, byte[] data) {
		for (int x = 0; x < data.length; x++)
			embedIntData(i + x * 8, j, data[x]);
	}

	public void embedIntArray(int x, int y, Cell[] cells, int start, int length) {
		for (int i = 0; i < length; i++)
			embedIntData(x + i * 8, y, cells[start + i].byteValue());
	}

	private void embedIntArray(int x, int y, int[] data, int start, int length) {
		for (int i = 0; i < length; i++)
			embedIntData(x + i * 8, y, (byte) data[start + i]);
	}

	public void embedIntSquareArray(int x, int y, byte[][] data) {
		for (int i = 0; i < data.length; i++)
			embedIntArray(x, y + i, data[i]);
	}

	public void embedIntSquareArray(int x, int y, Cell[][] cells, int startY,
			int startX, int lengthY, int lengthX) {
		for (int i = 0; i < lengthY; i++)
			embedIntArray(x, y + i, cells[startY + i], startX, lengthX);
	}

	public void embedIntSquareArray(int x, int y, int[][] data, int startY,
			int startX, int lengthY, int lengthX) {
		for (int i = 0; i < lengthY; i++)
			embedIntArray(x, y + i, data[startY + i], startX, lengthX);
	}

	public int extractData(int i, int j) {
		int color = getPixelGrayValue(i, j);
		if (Math.abs(color - 255) <= 1)
			return 0;
		else if (Math.abs(color - 0) <= 1)
			return 0;
		else
			return 1;
	}

	public byte extractIntData(int i, int j) {
		int[] bitArray = new int[8];
		for (int x = 0; x < 8; x++)
			bitArray[x] = extractData(i + x, j);
		return ConvertUtil.convertToByteFromBitArray(bitArray);
	}

	public int[] extractIntArray(int i, int j, int length) {
		int[] data = new int[length];
		for (int x = 0; x < length; x++)
			data[x] = extractIntData(i + x * 8, j);
		return data;
	}

	public int[][] extractIntSquareArray(int x, int y, int lengthY, int lengthX) {
		int[][] data = new int[lengthY][lengthX];
		for (int i = 0; i < lengthY; i++)
			data[i] = extractIntArray(x, y + i, lengthX);
		return data;
	}

}
